@@ -1,12 +1,15 @@
 import os
 import numpy as np
 
-from kgcnn.data.base import GraphDatasetBase
+from kgcnn.data.base import DownloadDataset, MemoryGraphDataset
+from kgcnn.utils.adj import add_edges_reverse_indices, make_adjacency_undirected_logical_or, \
+    precompute_adjacency_scaled, make_adjacency_from_edge_indices, convert_scaled_adjacency_to_list
 
 
-class CoraLUDataset(GraphDatasetBase):
+class CoraLUDataset(DownloadDataset, MemoryGraphDataset):
     """Store and process Cora dataset after Lu et al. 2003."""
 
+    dataset_name = "cora_lu"
     data_main_dir = os.path.join(os.path.expanduser("~"), ".kgcnn", "datasets")
     data_directory = "cora_lu"
     download_url = "https://linqs-data.soe.ucsc.edu/public/lbc/cora.tgz"
@@ -29,7 +32,13 @@ class CoraLUDataset(GraphDatasetBase):
         """
         self.class_label_mapping = None
         # Use default base class init()
-        super(CoraLUDataset, self).__init__(reload=reload, verbose=verbose)
+        self.length = 1
+
+        DownloadDataset.__init__(self, reload=reload, verbose=verbose)
+        MemoryGraphDataset.__init__(self, verbose=verbose)
+
+        if self.fits_in_memory:
+            self.read_in_memory(verbose=verbose)
 
     def read_in_memory(self, verbose=1):
         """Load Cora data into memory and already split into items.
@@ -64,19 +73,32 @@ class CoraLUDataset(GraphDatasetBase):
                                'Case_Based': 4,
                                'Probabilistic_Methods': 5,
                                'Neural_Networks': 6}
-        label_id = np.array([class_label_mapping[x] for x in labels], dtype=np.int)
-
-        self.nodes = nodes
-        self.edge_indices = indices
-        self.labels_node = label_id
         self.class_label_mapping = class_label_mapping
 
-        return self.nodes, self.edge_indices, self.labels_node, self.class_label_mapping
+        label_id = np.array([class_label_mapping[x] for x in labels], dtype=np.int)
+        labels = np.expand_dims(label_id, axis=-1)
+        labels = np.array(labels == np.arange(7), dtype=np.float)
 
-    def get_graph(self):
-        """Make graph tensor objects for Cora dataset after Lu et al. 2003.
+        self.node_attributes = [nodes[:, 1:]]
+        self.edge_indices = [indices]
+        self.edge_attributes = [np.ones_like(indices)[:, :1]]
+        self.node_labels = [labels]
+        self.node_number = [label_id]
+        self.graph_adjacency = make_adjacency_from_edge_indices(indices, np.ones_like(indices)[:, 0])
 
-        Returns:
-            tuple: nodes, indices, labels, class_label_mapping
-        """
-        return self.nodes, self.edge_indices, self.labels_node, self.class_label_mapping
+        return self
+
+    def make_undirected(self):
+        """Make edges undirected"""
+        self.graph_adjacency = make_adjacency_undirected_logical_or(self.graph_adjacency)
+        edi, ed = add_edges_reverse_indices(self.edge_indices[0], self.edge_attributes[0])
+        self.edge_indices = [edi]
+        self.edge_attributes = [ed]
+        return self
+
+    def scale_adjacency(self):
+        self.graph_adjacency = precompute_adjacency_scaled(self.graph_adjacency)
+        edi, ed = convert_scaled_adjacency_to_list(self.graph_adjacency)
+        self.edge_indices = [edi]
+        self.edge_attributes = [np.expand_dims(ed, axis=-1)]
+        return self
