@@ -2,6 +2,7 @@ import tensorflow as tf
 
 from kgcnn.layers.base import GraphBaseLayer
 from kgcnn.ops.partition import partition_row_indexing
+from kgcnn.ops.axis import get_positive_axis
 
 
 @tf.keras.utils.register_keras_serializable(package='kgcnn', name='GatherNodes')
@@ -22,7 +23,6 @@ class GatherNodes(GraphBaseLayer):
         self.axis = axis
 
     def build(self, input_shape):
-        """Build layer."""
         super(GatherNodes, self).build(input_shape)
         if len(input_shape) != 2:
             print("WARNING: Number of inputs for layer", self.name, "is expected to be 2.")
@@ -58,94 +58,43 @@ class GatherNodes(GraphBaseLayer):
                 return out
 
         # For arbitrary gather from ragged tensor use tf.gather with batch_dims=1.
-        out = tf.gather(inputs[0], inputs[1], batch_dims=1, axis=self.axis)  # Works in tf.__version__>=2.4
+        # Works in tf.__version__>=2.4
+        out = tf.gather(inputs[0], inputs[1], batch_dims=1, axis=self.axis)
         if self.concat_axis is not None:
             out = tf.concat([tf.gather(out, i, axis=self.concat_axis) for i in range(out.shape[self.concat_axis])],
                             axis=self.concat_axis)
         return out
 
     def get_config(self):
-        """Update config."""
         config = super(GatherNodes, self).get_config()
         config.update({"concat_axis": self.concat_axis, "axis": self.axis})
         return config
 
 
-@tf.keras.utils.register_keras_serializable(package='kgcnn', name='GatherNodesOutgoing')
-class GatherNodesOutgoing(GraphBaseLayer):
-    """Gather nodes by indices.
+@tf.keras.utils.register_keras_serializable(package='kgcnn', name='GatherNodesSelection')
+class GatherNodesSelection(GraphBaseLayer):
+    """Gather nodes with a defined index. E.g. for ingoing or outgoing nodes or angles.
 
-    For outgoing nodes, layer uses only index[1]. An edge is defined by index tuple (i,j) with i<-j connection.
-    If graphs indices were in 'batch' mode, the layer's 'node_indexing' must be set to 'batch'.
-    """
-
-    def __init__(self, axis=1, **kwargs):
-        """Initialize layer."""
-        super(GatherNodesOutgoing, self).__init__(**kwargs)
-        self.axis = axis
-
-    def build(self, input_shape):
-        """Build layer."""
-        super(GatherNodesOutgoing, self).build(input_shape)
-        if len(input_shape) != 2:
-            print("WARNING: Number of inputs for layer", self.name, "is expected to be 2.")
-        if [i for i in range(len(input_shape[0]))][self.axis] == 0:
-            print("WARNING: Shape error for", self.name, ", gather from batch-dimension is not intended.")
-
-    def call(self, inputs, **kwargs):
-        """Forward pass.
-
-        Args:
-            inputs (list): [nodes, tensor_index]
-
-                - nodes (tf.RaggedTensor): Node embeddings of shape (batch, [N], F)
-                - tensor_index (tf.RaggedTensor): Edge indices referring to nodes of shape (batch, [M], 2)
-
-        Returns:
-            tf.RaggedTensor: Gathered node embeddings that match the number of edges of shape (batch, [M], F)
-        """
-        # The primary case for aggregation of nodes from node feature list. Case from doc-string.
-        # Faster implementation via values and indices shifted by row-partition. Equal to disjoint implementation.
-        if all([isinstance(x, tf.RaggedTensor) for x in inputs]):
-            if all([x.ragged_rank == 1 for x in inputs]) and self.axis == 1:
-                # We cast to values here
-                node, node_part = inputs[0].values, inputs[0].row_splits
-                edge_index, edge_part = inputs[1].values, inputs[1].row_lengths()
-                indexlist = partition_row_indexing(edge_index, node_part, edge_part,
-                                                   partition_type_target="row_splits",
-                                                   partition_type_index="row_length",
-                                                   to_indexing='batch', from_indexing=self.node_indexing)
-                out = tf.gather(node, indexlist[:, 1], axis=0)
-                out = tf.RaggedTensor.from_row_lengths(out, edge_part, validate=self.ragged_validate)
-                return out
-
-        # For arbitrary gather from ragged tensor use tf.gather with batch_dims=1.
-        out = tf.gather(inputs[0], inputs[1][:, :, 1], batch_dims=1, axis=self.axis)  # Works in tf.__version__>=2.4
-        return out
-
-    def get_config(self):
-        """Update config."""
-        config = super(GatherNodesOutgoing, self).get_config()
-        config.update({"axis": self.axis})
-        return config
-
-
-@tf.keras.utils.register_keras_serializable(package='kgcnn', name='GatherNodesIngoing')
-class GatherNodesIngoing(GraphBaseLayer):
-    """Gather nodes by edge edge_indices.
-    
     For ingoing nodes, layer uses only index[0]. An edge is defined by index tuple (i,j) with i<-j connection.
-    If graphs indices were in 'batch' mode, the layer's 'node_indexing' must be set to 'batch'.
     """
 
-    def __init__(self, axis=1, **kwargs):
+    def __init__(self, selection_index, axis=1, axis_indices=2, **kwargs):
         """Initialize layer."""
-        super(GatherNodesIngoing, self).__init__(**kwargs)
-        self.axis= axis
+        super(GatherNodesSelection, self).__init__(**kwargs)
+        self.axis = axis
+        self.axis_indices = axis_indices
+
+        if not isinstance(selection_index, (list, tuple, int)):
+            raise ValueError("Indices for selection must be list or tuple for layer `GatherNodesSelection`.")
+
+        if isinstance(selection_index, int):
+            self.selection_index = [selection_index]
+        else:
+            self.selection_index = list(selection_index)
 
     def build(self, input_shape):
         """Build layer."""
-        super(GatherNodesIngoing, self).build(input_shape)
+        super(GatherNodesSelection, self).build(input_shape)
         if len(input_shape) != 2:
             print("WARNING: Number of inputs for layer", self.name, "is expected to be 2.")
         if [i for i in range(len(input_shape[0]))][self.axis] == 0:
@@ -161,12 +110,12 @@ class GatherNodesIngoing(GraphBaseLayer):
                 - tensor_index (tf.RaggedTensor): Edge indices referring to nodes of shape (batch, [M], 2)
 
         Returns:
-            tf.RaggedTensor: Gathered node embeddings that match the number of edges of shape (batch, [M], F)
+            list: Gathered node embeddings that match the number of edges of shape (batch, [M], F) for selection_index.
         """
         # The primary case for aggregation of nodes from node feature list. Case from doc-string.
         # Faster implementation via values and indices shifted by row-partition. Equal to disjoint implementation.
         if all([isinstance(x, tf.RaggedTensor) for x in inputs]):
-            if all([x.ragged_rank == 1 for x in inputs]) and self.axis == 1:
+            if all([x.ragged_rank == 1 for x in inputs]) and self.axis == 1 and self.axis_indices == 2:
                 # We cast to values here
                 node, node_part = inputs[0].values, inputs[0].row_splits
                 edge_index, edge_part = inputs[1].values, inputs[1].row_lengths()
@@ -175,21 +124,51 @@ class GatherNodesIngoing(GraphBaseLayer):
                                                    partition_type_index="row_length",
                                                    to_indexing='batch',
                                                    from_indexing=self.node_indexing)
-                out = tf.gather(node, indexlist[:, 0], axis=0)
-                out = tf.RaggedTensor.from_row_lengths(out, edge_part, validate=self.ragged_validate)
+                out = [tf.gather(node, tf.gather(indexlist, i, axis=1), axis=0) for i in self.selection_index]
+                out = [tf.RaggedTensor.from_row_lengths(x, edge_part, validate=self.ragged_validate) for x in out]
                 return out
 
         # For arbitrary gather from ragged tensor use tf.gather with batch_dims=1.
-        out = tf.gather(inputs[0], inputs[1][:, :, 0], batch_dims=1, axis=self.axis)  # Works in tf.__version__>=2.4
+        out = [tf.gather(inputs[0], tf.gather(inputs[1], i, axis=self.axis_indices), batch_dims=1, axis=self.axis) for i
+               in self.selection_index]  # Works in tf.__version__>=2.4
         return out
 
     def get_config(self):
-        """Update config."""
-        config = super(GatherNodesIngoing, self).get_config()
-        config.update({"axis": self.axis})
+        config = super(GatherNodesSelection, self).get_config()
+        config.update({"axis": self.axis, "axis_indices": self.axis_indices, "selection_index": self.selection_index})
         return config
 
 
+@tf.keras.utils.register_keras_serializable(package='kgcnn', name='GatherNodesOutgoing')
+class GatherNodesOutgoing(GatherNodesSelection):
+    """Gather nodes by indices.
+
+    For outgoing nodes, layer uses only index[1]. An edge is defined by index tuple (i,j) with i<-j connection.
+    If graphs indices were in 'batch' mode, the layer's 'node_indexing' must be set to 'batch'.
+    """
+
+    def __init__(self, selection_index=1, **kwargs):
+        super(GatherNodesOutgoing, self).__init__(selection_index=selection_index, **kwargs)
+
+    def call(self, inputs, **kwargs):
+        return super(GatherNodesOutgoing, self).call(inputs, **kwargs)[0]
+
+
+@tf.keras.utils.register_keras_serializable(package='kgcnn', name='GatherNodesIngoing')
+class GatherNodesIngoing(GatherNodesSelection):
+    """Gather nodes by edge edge_indices.
+    
+    For ingoing nodes, layer uses only index[0]. An edge is defined by index tuple (i,j) with i<-j connection.
+    If graphs indices were in 'batch' mode, the layer's 'node_indexing' must be set to 'batch'.
+    """
+
+    def __init__(self, selection_index=0, **kwargs):
+        super(GatherNodesIngoing, self).__init__(selection_index=selection_index, **kwargs)
+
+    def call(self, inputs, **kwargs):
+        return super(GatherNodesIngoing, self).call(inputs, **kwargs)[0]
+
+
 @tf.keras.utils.register_keras_serializable(package='kgcnn', name='GatherState')
 class GatherState(GraphBaseLayer):
     """Layer to repeat environment or global state for node or edge lists.
@@ -228,6 +207,5 @@ class GatherState(GraphBaseLayer):
         return out
 
     def get_config(self):
-        """Update config."""
         config = super(GatherState, self).get_config()
         return config
