@@ -2,7 +2,7 @@ import tensorflow as tf
 import tensorflow.keras as ks
 
 from kgcnn.layers.base import GraphBaseLayer
-
+from kgcnn.ops.axis import get_positive_axis
 
 # There are limitations for RaggedTensor working with standard Keras layers. Here are some simply wrappers.
 # This is a temporary solution until future versions of TensorFlow support more RaggedTensor arguments.
@@ -39,6 +39,7 @@ class Dense(GraphBaseLayer):
                                                     bias_constraint=bias_constraint)
 
     def call(self, inputs, **kwargs):
+        """Forward pass wrapping tf.keras layer."""
         # Call on a single Tensor
         if isinstance(inputs, tf.RaggedTensor):
             return tf.ragged.map_flat_values(self._kgcnn_wrapper_layer, inputs, **kwargs)
@@ -60,6 +61,7 @@ class Activation(GraphBaseLayer):
                                                                activity_regularizer=activity_regularizer)
 
     def call(self, inputs, **kwargs):
+        """Forward pass wrapping tf.keras layer."""
         if isinstance(inputs, tf.RaggedTensor):
             if inputs.ragged_rank == 1:
                 value_tensor = inputs.values  # will be Tensor
@@ -81,6 +83,7 @@ class Add(GraphBaseLayer):
         self._kgcnn_wrapper_layer = ks.layers.Add()
 
     def call(self, inputs, **kwargs):
+        """Forward pass wrapping tf.keras layer."""
         # Simply wrapper for self._kgcnn_wrapper_layer. Only works for simply element-wise operations.
         if all([isinstance(x, tf.RaggedTensor) for x in inputs]):
             # However, partition could be different, so this is only okay if ragged_validate=False
@@ -104,6 +107,7 @@ class Average(GraphBaseLayer):
         self._kgcnn_wrapper_layer = ks.layers.Average()
 
     def call(self, inputs, **kwargs):
+        """Forward pass wrapping tf.keras layer."""
         # Simply wrapper for self._kgcnn_wrapper_layer. Only works for simply element-wise operations.
         if all([isinstance(x, tf.RaggedTensor) for x in inputs]):
             # However, partition could be different, so this is only okay if ragged_validate=False
@@ -127,6 +131,7 @@ class Multiply(GraphBaseLayer):
         self._kgcnn_wrapper_layer = ks.layers.Multiply()
 
     def call(self, inputs, **kwargs):
+        """Forward pass wrapping tf.keras layer."""
         # Simply wrapper for self._kgcnn_wrapper_layer. Only works for simply element-wise operations.
         if all([isinstance(x, tf.RaggedTensor) for x in inputs]):
             # However, partition could be different, so this is only okay if ragged_validate=False
@@ -151,6 +156,7 @@ class Concatenate(GraphBaseLayer):
         self._kgcnn_wrapper_layer = ks.layers.Concatenate(axis=axis)
 
     def call(self, inputs, **kwargs):
+        """Forward pass wrapping tf.keras layer."""
         # Simply wrapper for self._kgcnn_wrapper_layer. Only works for simply element-wise operations.
         if all([isinstance(x, tf.RaggedTensor) for x in inputs]):
             # However, partition could be different, so this is only okay if ragged_validate=False
@@ -180,6 +186,7 @@ class Dropout(GraphBaseLayer):
         self._kgcnn_wrapper_layer = ks.layers.Dropout(rate=rate, noise_shape=noise_shape, seed=seed)
 
     def call(self, inputs, **kwargs):
+        """Forward pass wrapping tf.keras layer."""
         # Simply wrapper for self._kgcnn_wrapper_layer. Only works for simply element-wise operations.
         if isinstance(inputs, tf.RaggedTensor):
             if inputs.ragged_rank == 1:
@@ -218,6 +225,7 @@ class LayerNormalization(GraphBaseLayer):
             print("WARNING: This implementation only supports axis=-1 for RaggedTensors for now.")
 
     def call(self, inputs, **kwargs):
+        """Forward pass wrapping tf.keras layer."""
         if isinstance(inputs, tf.RaggedTensor):
             if self.axis == -1 and inputs.shape[-1] is not None and inputs.ragged_rank == 1:
                 value_tensor = inputs.values  # will be Tensor
@@ -264,6 +272,7 @@ class BatchNormalization(GraphBaseLayer):
             print("WARNING: This implementation only supports axis=-1 for RaggedTensors for now.")
 
     def call(self, inputs, **kwargs):
+        """Forward pass wrapping tf.keras layer."""
         if isinstance(inputs, tf.RaggedTensor):
             if self.axis == -1 and inputs.shape[-1] is not None and inputs.ragged_rank == 1:
                 value_tensor = inputs.values  # will be Tensor
@@ -277,4 +286,33 @@ class BatchNormalization(GraphBaseLayer):
     def get_config(self):
         config = super(BatchNormalization, self).get_config()
         config.update({"axis": self.axis})
-        return config
\ No newline at end of file
+        return config
+
+
+@tf.keras.utils.register_keras_serializable(package='kgcnn', name='ExpandDims')
+class ExpandDims(GraphBaseLayer):
+
+    def __init__(self,
+                 axis=-1,
+                 **kwargs):
+        """Initialize layer same as Activation."""
+        super(ExpandDims, self).__init__(**kwargs)
+        self.axis = axis  # We do not change the axis here
+
+    def call(self, inputs, **kwargs):
+        """Forward pass wrapping tf.keras layer."""
+        if isinstance(inputs, tf.RaggedTensor):
+            axis = get_positive_axis(self.axis, inputs.shape.rank + 1)
+            if axis > 1 and inputs.ragged_rank == 1:
+                value_tensor = inputs.values  # will be Tensor
+                out_tensor = tf.expand_dims(value_tensor, axis=axis-1)
+                return tf.RaggedTensor.from_row_splits(out_tensor, inputs.row_splits, validate=self.ragged_validate)
+            else:
+                print("WARNING: Layer", self.name, "fail call on values for ragged_rank=1, attempting keras call... ")
+        # Try normal keras call
+        return tf.expand_dims(inputs, axis=self.axis)
+
+    def get_config(self):
+        config = super(GraphBaseLayer, self).get_config()
+        config.update({"axis": self.axis})
+        return config
